/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.aspect;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;

/***********************************************************************************************************************
 *
 * This class is an aggregator of {@link Aspects}. It sequentially runs all the methods in the provided collection of
 * {@code Aspect}s.
 *
 * @author Fabrizio Giudici
 * @version $Id$
 * @experimental
 *
 **********************************************************************************************************************/
public class MultiAspect extends Aspect
  {
    @Nonnull
    private final Collection<Aspect> aspects;

    /*******************************************************************************************************************
     *
     * Creates an instance aggregating the given collection of  {@link Aspect}s.
     *
     * @param  aspects  the {@code Aspect}s
     *
     ******************************************************************************************************************/
    public MultiAspect (final @Nonnull Collection<Aspect> aspects)
      {
        this.aspects = new ArrayList<Aspect>(aspects);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void handleException (final @Nonnull Throwable t)
      {
        for (final @Nonnull Aspect aspect : aspects)
          {
            aspect.handleException(t);
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void runAfter()
      {
        for (final @Nonnull Aspect aspect : aspects)
          {
            aspect.runAfter();
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void runBefore()
      {
        for (final @Nonnull Aspect aspect : aspects)
          {
            aspect.runBefore();
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void runFinally()
      {
        for (final @Nonnull Aspect aspect : aspects)
          {
            aspect.runFinally();
          }
      }
  }
