/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
//@bluebook-begin 1
package it.tidalwave.netbeans.util;

import it.tidalwave.util.NotFoundException;
import javax.annotation.Nonnull;
import java.lang.reflect.Field;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public final class EnhancedLookups 
  {
    @Nonnull
    public static <T> T lookup (final @Nonnull Object object,
                                final @Nonnull Class<T> clazz, 
                                final @Nonnull T defaultValue)
      {
        T result = null;
        
        if (object instanceof Lookup)
          {
            result = ((Lookup)object).lookup(clazz);
          }
        
        if (object instanceof Lookup.Provider)
          {
            result = ((Lookup.Provider)object).getLookup().lookup(clazz);
          }
        
        return (result != null) ? result : defaultValue;
      }
//@bluebook-end 1    
//@bluebook-begin 2
    @Nonnull
    public static <T> T lookup (final @Nonnull Object object,
                                final @Nonnull Class<T> clazz)
      {
        try 
          {
            final Field defaultField = clazz.getDeclaredField("DEFAULT");
            return lookup(object, clazz, (T)defaultField.get(null));
          }
        catch (NoSuchFieldException e) 
          {
            throw new IllegalArgumentException("Class " + clazz + " doesn't provide a public static DEFAULT implementation", e);
          }
        catch (SecurityException e) 
          {
            throw new IllegalArgumentException("Class " + clazz + " doesn't provide a public static DEFAULT implementation", e);
          }
        catch (IllegalArgumentException e) 
          {
            throw new IllegalArgumentException("Class " + clazz + " doesn't provide a public static DEFAULT implementation", e);
          } 
        catch (IllegalAccessException e)
          {
            throw new IllegalArgumentException("Class " + clazz + " doesn't provide a public static DEFAULT implementation", e);
          }
      }
//@bluebook-end 2
    
    @Nonnull
    public static <T> T safeLookup (final @Nonnull Object object, 
                                    final @Nonnull Class<T> clazz)
      throws NotFoundException
      {
        T result = null;
        
        if (object instanceof Lookup)
          {
            result = ((Lookup)object).lookup(clazz);
          }
        
        if (object instanceof Lookup.Provider)
          {
            result = ((Lookup.Provider)object).getLookup().lookup(clazz);
          }
        
        return NotFoundException.throwWhenNull(result, "No instance of " + clazz + " in " + object);
      }
    
    //@bluebook-begin sugar
    @Nonnull
    public static Lookup createLookupWith (final @Nonnull Lookup lookup, final @Nonnull Object ... objects)
      {
        return new ProxyLookup(Lookups.fixed(objects), lookup);
      }
    
    @Nonnull
    public static Lookup createLookupWithout (final @Nonnull Lookup lookup, final @Nonnull Class<?> ... classes)
      {
        return Lookups.exclude(lookup, classes);  
      }
    
    @Nonnull
    public static <T> Lookup createLookupReplacing (final @Nonnull Lookup lookup, 
                                                    final @Nonnull Class<? super T> replacedClass, 
                                                    final @Nonnull T object)
      {
        return createLookupWith(createLookupWithout(lookup, replacedClass), object);
      }
    
    @Nonnull
    public static Lookup cloned (final @Nonnull Lookup lookup)
      {
        return new ProxyLookup(lookup);  
      }
    //@bluebook-end sugar
    
//@bluebook-begin 1
  }
//@bluebook-end 1
